#include <imsls.h>
#include <stdlib.h>
#include <stdio.h>
static void printErrorRates(int classification_errors[6], 
                            int n, char *label);
int main(){ 
   int i, j, k;
   int condPdfTableLength = 0;
   int n_patterns;  
   int n_variables;
   int n_sample        = 2000;
   int n_classes       =  2;      /* spam or no spam */
   int n_continuous    = 57;
   int *classErrors    = NULL;
   int *classification = NULL;
   int classSample[2000];
   int *predictedClass = NULL;
   int *rndSampleIndex = NULL;
   int classification_errors[6];
   float *continuous, *continuousSample;
   char* label1 = 
      "  Trainer from Training Dataset of %d Observations  \n";
   char* label2 = 
      "  Classifier for Entire Dataset of %d Observations  \n";   
   Imsls_f_nb_classifier *nb_classifier=NULL;
   float *spamData;
   int n_spam = 0;

   spamData = imsls_f_data_sets(11, IMSLS_N_OBSERVATIONS, &n_patterns, 
      IMSLS_N_VARIABLES, &n_variables, 0);

   continuous       = 
      (float*)malloc((n_patterns*n_continuous)*sizeof(float));
   continuousSample = 
      (float*)malloc((n_sample*n_continuous)*sizeof(float));
   classification   = (int*)malloc(n_patterns*sizeof(int));

   /* map continuous attributes into transformed representation */
   for(i=0; i<n_patterns; i++){
      for(j=0; j<n_continuous; j++) {
         if (j < 54 ) {
            continuous[i*(n_variables-1)+j] = (float)
               asin(sqrt( spamData[i*n_variables+j]/100));
         } else {
            continuous[i*(n_variables-1)+j] = 
               spamData[i*n_variables+j];
         }
      }
      classification[i] = (int)spamData[(i*n_variables)+n_variables-1];
      if(classification[i] == 1) n_spam++;
   }
   printf("Number of Patterns = %d Number Classified as Spam = %d \n\n", 
      n_patterns, n_spam);

   /* select random sample for training Naive Bayes Classifier */
   imsls_random_seed_set(1234567);
   rndSampleIndex=imsls_random_sample_indices(n_sample, n_patterns, 0);
   for(k=0; k<n_sample; k++){
      i = rndSampleIndex[k]-1;
      classSample[k] = classification[i];
      for(j=0; j<n_continuous; j++) {
         continuousSample[k*n_continuous+j] = 
            continuous[i*n_continuous+j];
      }
   }

   /* Train Naive Bayes Classifier */
   classErrors = imsls_f_naive_bayes_trainer(n_sample, n_classes, 
      classSample, 
      IMSLS_CONTINUOUS, n_continuous, continuousSample,
      IMSLS_NB_CLASSIFIER, &nb_classifier, 0);
   /* print error rates for training sample */
   printErrorRates(classErrors, n_sample, label1);

   /* CALL NAIVE_BAYES_CLASSIFICATION TO CLASSIFIY ENTIRE DATASET */
   predictedClass = imsls_f_naive_bayes_classification(nb_classifier, 
      n_patterns, 
      IMSLS_CONTINUOUS, continuous, 0);   
   /* calculate classification error rates for entire dataset */
   for(i=0; i<6; i++) classification_errors[i] = 0;
   for(i=0; i<n_patterns; i++){
      switch (classification[i])
      {
      case 0: 
         classification_errors[1]++;
         if(classification[i] != predictedClass[i]) 
            classification_errors[0]++;
         break;
      case 1:
         classification_errors[3]++;
         if(classification[i] != predictedClass[i]) 
            classification_errors[2]++;
         break;
      }
      classification_errors[5] =
         classification_errors[1]+classification_errors[3];
      classification_errors[4] = 
         classification_errors[0]+classification_errors[2];
   }
   /* print error rates for entire dataset */
   printErrorRates(classification_errors, n_patterns, label2);
}
static void printErrorRates(int classification_errors[6], 
                            int n, char *label)
{
   double p, p1, p0;
   p0 = 100.0*classification_errors[0]/classification_errors[1];
   p1 = 100.0*classification_errors[2]/classification_errors[3];
   p  = 100.0*classification_errors[4]/classification_errors[5];
   printf("     Classification Error Rates Reported by\n");
   printf(label, n);
   printf("----------------------------------------------------\n");
   printf("    Not Spam          Spam        |    TOTAL\n");
   printf(" %d/%d=%4.1f%%   %d/%d=%4.1f%%   | %d/%d=%4.1f%%\n", 
      classification_errors[0], classification_errors[1],
      p0, classification_errors[2], classification_errors[3], 
      p1, classification_errors[4], classification_errors[5], p);
   printf("----------------------------------------------------\n\n");
   return;
}                            
